* Motorola Flash Innovation 2003 Contest Project F193
*
* syncn.asm	for 68HC908QY4		Copyright 2003 Jed Margolin
************************************************************************
* Permission is granted to individuals to make copies for their own use.
* Any commercial use requires permission of the copyright holder.
************************************************************************
*
* NTSC Sync Generator for QY4 (Sync only, no HBlank or Color Burst)
*
* NTSC Sync with QY4
*
* Hardware: "NTSC Sync Generator"
*
* Uses MCU clock of 28.63636 MHz to produce Bus Clock of 7.15909 MHz which
* is twice Chroma (3.579545 MHz). 
*
* NTSC Horizontal rate is defined as Chroma * 2/455 = 15.734263 KHz.
*
#pagelength 65t
#pagewidth 124t
$base 10t

NVL  EQU	253	; 10 for testing VBI, 253 for real sync

$INCLUDE 'qy4equ.h'    	; Equates for MC68HC908QY4 MCU
*=====================================================
	ORG	RAMstart

vcount		RMB    1
vscount		RMB    1
vline		RMB    1
timer_vec	RMB    2   	; Timer Interrupt Redirection Address
*====================================================
* Video Signals
CSYNC	 	EQU 	0	; Composite Sync
VIDEO	 	EQU 	1	; 1 = On
nHVIDEO	 	EQU 	2	; 0 = Half Bright (not used) 
FIELD		EQU 	3	; 1 = Field 1
VBLANK 		EQU 	4	; V Blanking
SW 		EQU 	5	; Input (not used)
*====================================================
	ORG	$EE00

* First initialize any control registers and variables
START:  LDHX	#$0100		; locate the stack
	TXS

	RSP

	MOV	#$01,CONFIG1	; disable COP
*------------------------------------------------------------------------
* Clock Source
* External Crystal
	CLR	PORTA	
	BSET	4,DDRA		; PA4 output
	BSET	PA4,PORTA	; to help get crystal started
	MOV	#$3F,PAPUP	; Don't leave any inputs floating

	MOV	#$19,CONFIG2	; reset pin active, external crystal

	BSET	1,OSCSTAT
*------------------------------------------------------------------------
* Initialize I/O Ports
	CLR	PORTB
	CLR	DDRB		; Inputs
	MOV	#$FF,PBPUP	; Enable Port B Pullups 

	MOV	#$DF,PORTB	; HVID is off
	MOV	#$DF,DDRB
*------------------------------------------------------------------------
	BRA	MAIN
*------------------------------------------------------------------------
* NTSC Sync:
*
* Bus clock is twice chroma (3.579545 * 2 = 7.15909), which is why the 
* clock is 28.63636 (28.63636/4 = 7.15909)  
*
* Horizontal frequency is defined as chroma * 2/455 = 15.734263 KHz
* Timer Interrupts can be set as an integer number of bus clocks, so it
* is set to 455 cycles.
*
* Everything is done at the interrupt level.
*
* The main line code is just NOPs because they are single cycle instructions.
*
* Some things are done at twice the horizontal rate and require cycle 
*   counting.
*
* Having 455 CPU cyles per Horizontal Line makes it impossible to divide
*   the H Line exactly in half for EQ and VSync pulses. 
*
* Interlacing is done as per NTSC spec by moving every other VBI 1/2H early
*
MAIN:	MOV	#NVL,vcount  ; number of active video lines/field
			     ; 10 for testing, 253 for real sync

	MOV	#9,vline     ; number of lines in Vertical Blanking Interval

	LDHX	#TIRQ1	     ; Timer Overflow Vector: Real Horizontal Sync
	STHX	timer_vec

	CLRA
	JSR	TIRQ_Init	; Initialize timer irqs

	JMP	NOPS		; Everything gets done in the timer interrupt 
*=============================================================================
* Initialize Timer Interrupts
TIRQ_Init:

* Timer counter, must write to high byte first, 455 -> 0-454
	MOV	#$01,TMODH		; 454 = $01C6
	MOV	#$C6,TMODL

	MOV	#$00,TSC		; PS2, PS1, PS0; TSTOP = 0

	BSET	TOIE,TSC	; enable interrupts

	CLI

	RTS
*==========================================================================
* Timebase 						($021C)
* Jump to Address in User's Vector Table
TIM_INT:
*        PSHX		; we don't care about trashing XRegs
*        PSHH

	LDHX	timer_vec
	JMP	,X
*==========================================================================
UNUSED: RTI
*==========================================================================
***************************************************************************
* Timer Overflow Interrupt (TIM_OF_VEC)
* Active Video: Field 2 begins on Line 271 1/2, ends on Line 525
* Real Horizontal Sync
TIRQ1:	

*---------
* HSYNC will be 4.77 us = 34 cycles
	BSET	CSYNC,PORTB	; [4]  SYNC on

	BCLR	TOF,TSC		; [4]  reset interrupt flag

	LDX	#8		; [2]
tq1a:	DBNZX	tq1a		; [3]
	BCLR	CSYNC,PORTB	; [4]  SYNC off
* ---------------------------------------------------
* Check Vertical Count
	DBNZ	vcount,tq1c

* We have reached the end of active video. Next, start VBI
	LDHX	#TIRQ2	 ; Timer Overflow Vector: Start Field 1 VBI next time
	STHX	timer_vec

	BSET	VBLANK,PORTB	; oscilloscope trigger high
	BSET	FIELD,PORTB	; Field 1 flag

	BRA	TIRQRET

* Video is still active. We have about 50 us to do something else
tq1c:	JSR	Do_Field2
	BRA	TIRQRET
*======================================================================
NOPORG	EQU	$FC30
NOPSHI	EQU	{NOPORG > 8}	; high byte of .ORG for nops.h
NOPSLO	EQU	NOPORG&$FF	; low byte of .ORG for nops.h
*---------
* RTI to beginning of NOPS to make sure an interrupt does not occur during
* the 2-cycle JMP instruction.
TIRQRET:
	TSX			; 2	SP to H:X

	LDA	#NOPSHI		; 2	NOPS High Address
	STA	3,X		; 4

	LDA	#NOPSLO		; 2	NOPS Low Address
	STA	4,X		; 4

	RTI			; 7	(21)
*==========================================================================
* Field 1 VBI: Leading Equalization Pulses
TIRQ2:	

* Leading Equalization Pulses
* We need to do two equalization pulses per line
*
* we need to align it with HSync
* Line 1
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
	NOP			; [1]
* 
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1	

* end at 455 cycles
* Line 2
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
	NOP
* 
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1	

* end at 455 cycles
* Line 3
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
	NOP
* 
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1	

* end at 455 cycles
* Field 1 VBI: Vertical Sync Pulses
* We need to do two VSync pulses per line
* Line 4
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1
	NOP

	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1

* end at 455 cycles
* We need to do two VSync pulses per line
* Line 5
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1
	NOP

	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1

* end at 455 cycles
* We need to do two VSync pulses per line
* Line 6
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1
	NOP

	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1

* end at 455 cycles
* Trailing Equalization Pulses
* We need to do two equalization pulses per line
* Line 7
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
	NOP
* 
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1	

* end at 455 cycles
* We need to do two equalization pulses per line
* Line 8
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
	NOP
* 
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1	

* end at 455 cycles
* We need to do two equalization pulses per line
* Line 9
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
	NOP
* 
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ2	
	BCLR	TOF,TSC		; [4] reset interrupt flag

	LDHX	#TIRQ3	    	; Timer Overflow Vector: active video
	STHX	timer_vec
	MOV	#NVL,vcount ; number of vertical lines: 10 test, 253 real sync
	MOV	#9,vline  ; number of lines used in Vertical Blanking Interval

	JMP	TIRQRET
*======================
* Active Video: Field 1 begins on Line 10, ends on Line 262 1/2
* Real Horizontal Sync
TIRQ3:	

*---------
* HSYNC will be 4.77 us = 34 cycles
	BSET	CSYNC,PORTB	; [4]  SYNC on

	BCLR	TOF,TSC		; [4]  reset interrupt flag

	LDX	#8		; [2]
tq3a:	DBNZX	tq3a		; [3]
	BCLR	CSYNC,PORTB	; [4]  SYNC off
* ---------------------------------------------------
* Check Vertical Count
	DBNZ	vcount,tq3c

* We have reached the end of active video. Next, start VBI
	LDHX	#TIRQ4	     ; Timer Overflow Vector: Start VBI next time
	STHX	timer_vec

	BSET	VBLANK,PORTB	; oscilloscope trigger high
	JMP	TIRQRET

* Video is still active
* We have about 50 us to do something else
tq3c:	JSR	Do_Field1
	JMP	TIRQRET
*========================================================================
* Active Video: end of Field 1
* Start Field 2 VBI 
* Do 1/2 Horizontal line and 1 Equalizing pulse
* Real Horizontal Sync
TIRQ4:	

* Line 262
*---------
* HSYNC will be 4.77 us = 34 cycles
	BSET	CSYNC,PORTB	; [4]  SYNC on

	BCLR	TOF,TSC		; [4]  reset interrupt flag

	LDX	#8		; [2]
tq4a:	DBNZX	tq4a		; [3]
	BCLR	CSYNC,PORTB	; [4]  SYNC off
* ---------------------------------------------------
* Bring cycle count from CSYNC to 228 (455/2 or so)
	LDX	#61
tq4b:	DBNZX	tq4b
	NOP
	BCLR	FIELD,PORTB	; [4] Field 1 flag
*---------
* Do 1 Equalizing Pulse
* Do the pulse and get ready for the next step
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ2

	LDHX	#TIRQ5	    ; Timer Overflow Vector: active video
	STHX	timer_vec

	JMP	TIRQRET
*======================
* Field 2 VBI: Equalization Pulses
TIRQ5:	
* Line 263
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
	NOP			; [1]
* 
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1	

* end at 455 cycles
* Line 264
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
	NOP			; [1]
* 
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1	

* end at 455 cycles
* Line 265
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
	NOP			; [1]

* end at 228 cycles (6 EQ pulses)
* Field 2 VBI: Vertical Sync Pulses
* We need to do two VSync pulses per line
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1

* Line 266
* end at 455 cycles
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1
	NOP			; [1]

* end at 228 cycles
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1

* Line 267
* end at 455 cycles
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1
	NOP			; [1]

	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1

* end at 455 cycles
* Line 268
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1
	NOP			; [1]

* end at 228 cycles
* Trailing Equalization Pulses
* We need to do two equalization pulses per line
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
* 
* end at 455 cycles
* Line 269
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1	
	NOP			; [1]

	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
* 
* end at 455 cycles
* Line 270
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1	
	NOP			; [1]

	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
* 
* end at 455 cycles
* Line 271
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ2	
	BCLR	TOF,TSC		; [4] reset interrupt flag

* Start of Field 2 Video
	LDX	#3		; [2]
tq5a:	DBNZX	tq5a		; [3]
	NOP			; [1]

	BCLR	CSYNC,PORTB	; [4]  SYNC off

* Wait 
	LDX	#66	  ; [2]
tq5b:	DBNZX	tq5b	  ; [3]
* Line 271 1/2
*---------
* End of trailing equalization pulses
	BCLR	VBLANK,PORTB	; oscilloscope trigger low
	MOV	#9,vline
*------------
* Start Field 2 Active Video, which ends in an integer number of H lines
* and begins Field 1 VBI
	LDHX	#TIRQ1		; Timer Overflow Vector
	STHX	timer_vec

	MOV	#NVL,vcount ; number of vertical lines: 10 test, 253 real sync
	MOV	#9,vline  ; number of lines used in Vertical Blanking Interval

	JMP	TIRQRET
*============================================================================
***************************************************************************
*==========================================================================
* Wait 1000 us (for testing the clock frequency)
*   cycles = 19 + 8*X ; X = (cycles-19)/8
*
* At 7.159 MHz, we need 1000*7.159 = 7159 cycles
* xval = (time*BClock -19)/8
* time = (xval*8 +19)/BClock
* For BClock = 7.159 MHz, xval = 893
* At 2.4576 MHz, time for xval = 893 is 2915 us

*			JSR		4
WAIT1ms:
        PSHX		;		2
        PSHH		;		2

	LDHX	#893	; 		3

wt1ms	AIX	#-1	;		2
	CPHX	#0000	;		3
	BNE	wt1ms   ;		3

	PULH		;		2
	PULX		;		2
	RTS		;		4
*=============================================================================
TEST1:	BSET	CSYNC,PORTB
	JSR	WAIT1ms
	BCLR	CSYNC,PORTB
	JSR	WAIT1ms
	BRA	TEST1
*=============================================================================
*=============================================================================
Do_Field1:
	JSR	Box
	INC	vline
	RTS
*==========================================================================
Do_Field2:
	JSR	Box
	INC	vline
	RTS
*==========================================================================
*==========================================================================
Box:	LDA	vline
	CMP	#100
	BLO	bxout

	CMP	#175
	BHI	bxout

* Center the Box
	LDHX	#47
bx1:	DBNZX	bx1
*---------
* Start: 212 cycles from Sync Low
	BSET 	VIDEO,PORTB

	LDX	#30
bx2:	DBNZX	bx2

	BCLR 	VIDEO,PORTB
* End
*---------
bxout:	RTS
*==========================================================================
* Equalization Pulse
EQ1:	
*	BSET	CSYNC,PORTB	; [4]  SYNC on

	LDX	#2		; [2]
eq1a:	DBNZX	eq1a		; [3]
	NOP			; [1]
	BCLR	CSYNC,PORTB	; [4]  SYNC off

* Wait 
	LDX	#66		; [2]
eq1b:	DBNZX	eq1b	   	; [3]
	NOP

	RTS			; [4]
*==========================================================================
* Equalization Pulse
EQ2:	
*	BSET	CSYNC,PORTB	; [4]  SYNC on

	LDX	#1		; [2]
eq2a:	DBNZX	eq2a		; [3]
	NOP			; [1]
	NOP			; [1]

	BCLR	CSYNC,PORTB	; [4]  SYNC off

	RTS			; [4]
*==========================================================================
* VSync pulse
VS1:	
*	BSET	CSYNC,PORTB	; [4]  SYNC on

	LDX	#61
vs1a:	DBNZX	vs1a	   	; [3]	
	NOP			; [1]

	BCLR	CSYNC,PORTB	; [4]  SYNC off
	
	LDX	#7		; [3]  {7}
vs1b:	DBNZX	vs1b	   	; [3]	
	NOP			; [1]
	RTS
*==========================================================================
******************************************************************************
* NOP Table
******************************************************************************
*==========================================================================
* Put NOPs here
	ORG	NOPORG
$INCLUDE 'nops1.h'   ; 456 NOPs (NOPSHI and NOPSLO must match this address)
*==========================================================================
******************************************************************************
* Vector Table
	ORG	ADCVEC		; A/D conversion complete		
	FDB	UNUSED

	ORG	KBVEC	 	; Keyyboard Vector			
	FDB	UNUSED

	ORG	TIM_OF_VEC  	; Timer Overflow Vector		
	FDB	TIM_INT

	ORG	TIM_CH1_VEC 	; Timer Channel 1 Vector		
	FDB	UNUSED

	ORG	TIM_CH0_VEC	; Timer Channel 2 Vector		
	FDB	UNUSED

	ORG	IRQ_VEC		; IRQ Vector				
	FDB	UNUSED

	ORG	SWI_VEC		; Software Interrupt			
	FDB	UNUSED

	ORG 	RESET_VEC
	FDB	START
*=============================================================================
