* Motorola Flash Innovation 2003 Contest Project F193
*
* vtrig1.asm	Video Line Trigger  MCU 1    Copyright 2003 Jed Margolin
************************************************************************
* Permission is granted to individuals to make copies for their own use.
* Any commercial use requires permission of the copyright holder.
************************************************************************
*
* Counts video lines from an external NTSC video source	and produces a pulse 
* to trigger an oscilloscope at the selected line(s). 
*
* Modes: 1. Single trigger line.
*        2. Two trigger lines with a level output for dual channel 
*	    oscilloscopes with the ability to add the two channels, and having
*	    an external trigger input.
*
* Hardware: "Video Line Trigger for Oscilloscope"
*
#pagelength 65t
#pagewidth 124t
$base 10t

$INCLUDE 'qy4equ.h'    	; Equates for MC68HC908QY4 MCU
*====================================================
* Port B
CSYNC	 	EQU 	0	; In: Composite Sync
VSYNC	 	EQU 	1	; In: Vertical Sync
FIELD		EQU 	2	; In: 0 = Field 1
TRIGGER	 	EQU 	3	; Out: 0 = Oscilloscope trigger
CHANNEL2	EQU 	4	; Out: Channel 2 for line offset
C2En		EQU 	5	; In: 0 = Enable Channel 2 offset
*NC		EQU 	6	; In: not used
Link_Dat	EQU 	7	; In: Link Data

* Port A
Link_Clk	EQU 	1	; In: Link Clock
*====================================================
	ORG	RAMstart
vline:	  	RMB    2
line1:	  	RMB    2	; line 1
line2:	  	RMB    2	; line 2
linkdata:	RMB    2	; link data received
vbi_flag	RMB    1
*====================================================
*	ORG	ROMstart
	ORG	$F000
* First initialize any control registers and variables
START:  LDHX	#$0100	; locate the stack
	TXS

	RSP

	MOV	#$01,CONFIG1	; disable COP
*------------------------------------------------------------------------
* Clock Source
* External Crystal
	CLR	PORTA	
	BSET	4,DDRA		; PA4 output
	BSET	PA4,PORTA	; to help get crystal started
	MOV	#$3F,PAPUP	; Don't leave inputs floating

	MOV	#$19,CONFIG2	; reset pin active, external crystal

	BSET	1,OSCSTAT
*------------------------------------------------------------------------
* Initialize I/O Ports
	CLR	DDRA	; Inputs
	CLR	DDRB	; Inputs

	MOV	#$FF,PBPUP	; Enable Port B Pullups 

*	BRA	vlt

* Enable d1 PORTA keyboard interrupt
	BSET	KBIE1,KBIER	; enable d1
	BCLR	KBIE1,DDRA	; make it an input so we can read it

	BSET	MODEK,KBSCR	; pin must return to 1 to reset to re-enable
*	BCLR	MODEK,KBSCR	; edged triggered interrupt

	CLI
*------------------------------------------------------------------------
* Video Line Comparator
vlt:	BCLR	VSYNC,DDRB	; /VSYNC Input
	BCLR	CSYNC,DDRB	; /CSYNC Input

	BSET	TRIGGER,PORTB	; /TRIGGER Out
	BSET	TRIGGER,DDRB	; TRIGGER Out

	BCLR	CHANNEL2,PORTB	; /Channel 2 Flag
	BSET	CHANNEL2,DDRB	; Channel 2 Flag

	LDHX	#$00
	STHX	vline
	STX	vbi_flag

	LDHX	#21		; closed captioning 1
	STHX	line1

	LDHX	#284		; closed captioning 2
	STHX	line2
*--------------------
* main loop
vltstart:

* Wait for /VSYNC Low transition
vltbr1	BRCLR	VSYNC,PORTB,vltbr1	; 5 cycles = .625 us
vltbr2	BRSET	VSYNC,PORTB,vltbr2	; 5 cycles = .625 us

* We are at the Leading Edge of /VSYNC 
* Give the LM1881 time for the Field index to become valid
	JSR	WAIT10us

* Field 1 or Field 2
	BRCLR	FIELD,PORTB,vltstart   ; Go back and wait for start of Field 1

* Get past the VBI serrations
	LDHX	#0007
vt1:	BSR	WaitSync
	DBNZX	vt1

* During VBI, Sync pulses happen twice per line.
*
* We will start at line 9
* (We are one ahead because the trigger is gated by the next sync.)
	LDHX	#9
	STHX	vline

* Ready to look at vlines
vtloop:
	LDHX	vline
	CPHX	#10		; Field 1 Active video starts at line 10
	BHI	vbi1		; we are out of Field 1 vbi
* vline .LT. 10
	BSET	0,vbi_flag	; we are in Field 1 vbi
	BRA	vtrig1

* vline .GE. 10
vbi1:	CPHX	#263		; Field 1 Active video ends at line 262 1/2
	BHI	vbi2
* vline .LE. 263
	BCLR	0,vbi_flag	; we are in Field 1 active video
	BRA	vtrig1

* vline .GT. 263
vbi2:	CPHX	#272		; Field 2 vbi ends at line 272
	BHI	vbi3
* vline .LT. 272
	BSET	0,vbi_flag	; we are in Field 2 vbi
	BRA	vtrig1

* vline .GE. 272
vbi3:	BCLR	0,vbi_flag	; we are in Field 2 active video

* Ready to look at the lines
vtrig1:
	BSR	WaitSync
	BSET	TRIGGER,PORTB		; Reset the Trigger
	
* If we are in vbi do an extra sync
	BRCLR	0,vbi_flag,vtrig2
	BSR	WaitSync
	BCLR	0,vbi_flag		; reset the vbi flag

vtrig2:	LDHX	vline
	CPHX	line1
	BNE	vtrig3

	BSET	CHANNEL2,PORTB		; Channel 2 Flag
	BCLR	TRIGGER,PORTB		; Trigger

vtrig3:	CPHX	line2
	BNE	vtrig4

	BRSET	C2En,PORTB,vtrig4	; Channel 2 enabled?
	BCLR	CHANNEL2,PORTB		; Channel 2 Flag
	BCLR	TRIGGER,PORTB		; Trigger

vtrig4:	LDHX	vline
	AIX	#1
	STHX	vline

	CPHX	#527
	BNE	vtloop

	BRA	vltstart
*=====================================================================
* Wait for Composite Sync Pulse
WaitSync:
ws1	BRSET	CSYNC,PORTB,ws1		; 5 cycles = .625 us
ws2	BRCLR	CSYNC,PORTB,ws2		; 5 cycles = .625 us
	RTS				; 6 cycles = .750 us
*----------------
* xval = (time*8.0 -16)/3		(8.0 MHz)
*      = (10 * 8 - 1) /3 = 64/3 = 21
* time = (21*3 +16)/8 = 9.9 us
* 	JSR				5
WAIT10us:
        PSHA		;		2

	LDA	#21	; 		3
wt10	DBNZA	wt10    ;		3	

	PULA		;		2
	RTS		;		4
*=============================================================================
* Wait 1000 us
*   cycles = 19 + 8*X ; X = (cycles-19)/8
*
* At 8.0 MHz, we need 1000*8.0 = 8000 cycles
* xval = (time*BClock -19)/8
* time = (xval*8 +19)/BClock
* For BClock = 8.0 MHz, xval = 997
*			JSR		4
WAIT1ms:
        PSHX		;		2
        PSHH		;		2

	LDHX	#997	; 		3

wt1ms	AIX	#-1	;		2
	CPHX	#0000	;		3
	BNE	wt1ms   ;		3

	PULH		;		2
	PULX		;		2
	RTS		;		4
*=============================================================================
MSG0:  FCB  $80,'Sync 1   ',$FF
*=============================================================================
* In case of spurious interrupt
UNUSED: RTI
*=============================================================================
* Comm link from MCU2
* Use Data and Clock, Data is set up before rising edge of Clock
* ___	         __     __     __     __     __     __     __     ________
*    \________d0/  \_d1/  \_d2/  \_d3/  \_d4/  \_d5/  \_d6/  \ d7/ 

LINKIRQ:	

* We got the start bit for the comm link from MCU2 
	BSET	IMASKK,KBSCR		; Mask KB interrupts until we are done

	CLRA
	LDX	#8			; get 8 bits

lq1:	BRCLR	Link_Clk,PORTA,lq1	; wait for Link_Clk to go high

* Data should be valid
	BRCLR	Link_Dat,PORTB,lq2	; sets the Carry Bit with link data
lq2:	RORA

lq3:	BRSET	Link_Clk,PORTA,lq3	; wait for Link_Clk to go low
	
	DBNZX	lq1

* We have the first byte (high byte)
	STA	linkdata
*--------
* Wait for the Start bit for the second byte
	CLRA
	LDX	#8			; get 8 bits

lq4:	BRSET	Link_Clk,PORTA,lq4	; wait for Link_Clk to go low
lq5:	BRCLR	Link_Clk,PORTA,lq5	; wait for Link_Clk to go high

* Data should be valid
	BRCLR	Link_Dat,PORTB,lq6	; sets the Carry Bit with link data
lq6:	RORA

	DBNZX	lq4

* We have the second byte (low byte)
	STA	linkdata+1

* Check if it is line1 or line2
	BRSET	2,linkdata,lq7		
	LDHX	linkdata
	STHX	line1
	BRA	lqout

* line2
lq7:	BCLR	2,linkdata
	LDHX	linkdata
	STHX	line2

lqout:	RSP
	BSET	ACKK,KBSCR	; Clear any KB interrupts
	BCLR	IMASKK,KBSCR	; Enable KB interrupts again

	CLI
	JMP	vltstart
*=============================================================================
******************************************************************************
* Vector Table
	ORG	ADCVEC		; A/D conversion complete		
	FDB	UNUSED

	ORG	KBVEC	 	; Keyboard Vector			
	FDB	LINKIRQ

	ORG	TIM_OF_VEC  	; Timer Overflow Vector		
	FDB	UNUSED

	ORG	TIM_CH1_VEC 	; Timer Channel 1 Vector		
	FDB	UNUSED

	ORG	TIM_CH0_VEC	; Timer Channel 2 Vector		
	FDB	UNUSED

	ORG	IRQ_VEC		; IRQ Vector				
	FDB	UNUSED

	ORG	SWI_VEC		; Software Interrupt			
	FDB	UNUSED

	ORG 	RESET_VEC
	FDB	START
*=============================================================================
