* Motorola Flash Innovation 2003 Contest Project F193
*
* vtrig2.asm	Video Line Trigger  MCU 2    Copyright 2003 Jed Margolin
************************************************************************
* Permission is granted to individuals to make copies for their own use.
* Any commercial use requires permission of the copyright holder.
************************************************************************
*
* Mode 1: User selects line 1 and line 2 triggers, displayed on-screen in
*         video. Selection is sent to MCU1.
* Mode 2: User selects line 1 trigger. Selection is sent to MCU1.
*
* Hardware: "Video Line Trigger for Oscilloscope"
*
* QY4 Clock is 32MHz with 508 cycles/Horizontal line.
* HSync is 15.748 KHz. (8 MHz / 508)
*
#pagelength 65t
#pagewidth 124t
$base 10t

NVL  	  EQU	253	; 10 for testing VBI, 253 for real sync

$INCLUDE 'qy4equ.h'    	; Equates for MC68HC908QY4 MCU
*=====================================================
	ORG	RAMstart

vcount		RMB    1
vline		RMB    1
vlfunction	RMB    2
chline		RMB    1
ch0address	RMB    2	; character 0 address
ch1address	RMB    2	; character 1 address
ch2address	RMB    2	; character 2 address
ch3address	RMB    2	; character 3 address
ch4address	RMB    2	; character 4 address
ch5address	RMB    2	; character 5 address
ch6address	RMB    2	; character 6 address
chtimes		RMB    1	; number of lines to repeat/character line
line1:		RMB    2	; line trigger 1
line1bcd:	RMB    2	; line trigger 1
line2:		RMB    2	; line trigger 2
line2bcd:	RMB    2	; line trigger 2
linkdata:	RMB    2	; link data to send
timecount:	RMB    1
ch0adstor	RMB    2	; character 0 address storage
ch1adstor	RMB    2	; character 1 address storage
ch2adstor	RMB    2	; character 2 address storage
ch3adstor	RMB    2	; character 3 address storage
ch4adstor	RMB    2	; character 4 address storage
ch5adstor	RMB    2	; character 5 address storage
timer_vec	RMB    2   	; Timer Interrupt Redirection Address
sw1		RMB    3	; increment switch processing
sw2		RMB    3	; increment switch processing
sw3		RMB    3	; select next field
fsel		RMB    1	; d0 = which line number to change
				; d7: 1 = new data
bin		RMB    2	; for bin_bcd conversion
bcd		RMB    2	; for bin_bcd conversion
*====================================================
* Video Signals
CSYNC	 	EQU 	0	; Output	Composite Sync
VIDEO	 	EQU 	1	; Output	1 = On
SelSW		EQU 	2	; Input		Select Field
IncSW		EQU 	3	; Input		0 = Increment Switch
DecSW		EQU 	4	; Input		0 = Decrement Switch
C2En		EQU 	5	; Input 	0 = Channel 2 enabled
Link_Dat	EQU 	6	; Output	Link Data
Link_Clk	EQU 	7	; Output	Link Clock
*====================================================
	ORG	$EE00

* First initialize any control registers and variables
START:  LDHX	#$0100	; locate the stack
	TXS

	RSP

	MOV	#$01,CONFIG1	; disable COP
*------------------------------------------------------------------------
* Clock Source
* External Crystal
	CLR	PORTA	
	BSET	4,DDRA		; PA4 output
	BSET	PA4,PORTA	; to help get crystal started
	MOV	#$3F,PAPUP	; Leave no inputs floating

	MOV	#$19,CONFIG2	; reset pin active, external crystal

	BSET	1,OSCSTAT
*------------------------------------------------------------------------
* Initialize I/O Ports
	CLR	PORTB
	CLR	DDRB		; Inputs
	MOV	#$FF,PBPUP	; Enable Port B Pullups 

	MOV	#$C3,PORTB	; Link_Clk is high
	MOV	#$C3,DDRB

	LDHX	#$FF
	STHX	sw1
	STHX	sw2
	STHX	sw3
	CLR	sw1+2
	CLR	sw2+2
	CLR	sw3+2
	CLR	fsel
*------------------------------------------------------------------------
	BRA	MAIN
*------------------------------------------------------------------------
* NTSC Sync:
*
* Bus Clock is 8.0000 MHz, cycle = 0.125 us  
*
* Horizontal frequency is defined as 508 cycles = 63.5 us = 15.748 KHz
*
* HSync = 4.80 us = 38 clocks
*
* The Interrupts can be set as an integer number of bus clocks.
*
* Everything is done at the interrupt level.
*
* The main line code is just NOPs because they are single cycle instructions.
*
* Some things are done at twice the horizontal rate and require cycle 
* counting.
*
* Interlacing is done as per NTSC spec by moving every other VBI 1/2H early
*
MAIN:	MOV	#NVL,vcount  ; number of active video lines/field
			     ; 10 for testing, 253 for real sync

	MOV	#9,vline     ; number of lines in Vertical Blanking Interval
	MOV	#8,chline

	LDHX	#21		; default line 1
	STHX	line1
	LDHX	#$0021		; default line 1
	STHX	line1bcd

	LDHX	#284		; default line 2
	STHX	line2
	LDHX	#$0284		; default line 2
	STHX	line2bcd

* Initialize display character addresses
	JSR	Get_CH		; get character addresses

	CLR	chtimes
	MOV	#30,timecount	; 30 fields/sec for the clock

	LDHX	#TIRQ1		; Timer Overflow Vector: Real Horizontal Sync
	STHX	timer_vec

	LDHX	#vlf1		; the first video line function
	STHX	vlfunction

	CLRA
	JSR	TIRQ_Init	; Initialize timer irqs

	JMP	NOPS		; Everything gets done in the timer interrupt 
*=============================================================================
* Initialize Timer Interrupts
TIRQ_Init:

* Timer counter, must write to high byte first, 507 -> 0-507 (508 cycles)
	LDA	#$01		; 507 = $01FB
	STA	TMODH		

	LDA	#$FB
	STA	TMODL

	LDA	#$00		; PS2, PS1, PS0; TSTOP = 0
	STA	TSC

	BSET	TOIE,TSC	; enable interrupts

	CLI

	RTS
*==========================================================================
* Timebase 						($021C)
* Jump to Address in User's Vector Table
TIM_INT:
*        PSHX		; we don't care about trashing XRegs
*        PSHH

	LDHX	timer_vec
	JMP	,X
*==========================================================================
UNUSED: RTI
*==========================================================================
***************************************************************************
* Timer Overflow Interrupt (TIM_OF_VEC)
* Active Video: Field 2 begins on Line 271 1/2, ends on Line 525
* Real Horizontal Sync
TIRQ1:	

*---------
* HSYNC will be 4.77 us = 38 cycles
	BSET	CSYNC,PORTB	; [4]  SYNC on
	BCLR	TOF,TSC		; [4]  reset interrupt flag

	LDX	#9		; [2]
tq1a:	DBNZX	tq1a		; [3]
	NOP			; [1]

	BCLR	CSYNC,PORTB	; [4]  SYNC off
* ---------------------------------------------------
* Check Vertical Count
	DBNZ	vcount,tq1c

* We have reached the end of active video. Next, start VBI
	LDHX	#TIRQ2	 ; Timer Overflow Vector: Start Field 1 VBI next time
	STHX	timer_vec

	BRA	TIRQRET

* Video is still active. We have about 50 us to do something else
tq1c:	JSR	Do_Video
	BRA	TIRQRET
*======================================================================
NOPORG	EQU	$FC30
NOPSHI	EQU	{NOPORG > 8}	; high byte of .ORG for nops.h
NOPSLO	EQU	NOPORG&$FF	; low byte of .ORG for nops.h
*---------
* RTI to beginning of NOPS to make sure an interrupt does not occur during
* the 2-cycle JMP instruction.
TIRQRET:
	TSX			; 2	SP to H:X

	LDA	#NOPSHI		; 2	NOPS High Address
	STA	3,X		; 4

	LDA	#NOPSLO		; 2	NOPS Low Address
	STA	4,X		; 4

	RTI			; 7	(21)
*==========================================================================
* Field 1 VBI: Leading Equalization Pulses
TIRQ2:	

* Leading Equalization Pulses
* We need to do two equalization pulses per line
*
* we need to align it with HSync
* Line 1
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
* 
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1	

* end at 508 cycles
* Line 2
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
* 
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1	

* end at 508 cycles
* Line 3
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
* 
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1	

* end at 508 cycles
* Field 1 VBI: Vertical Sync Pulses
* We need to do two VSync pulses per line
* Line 4
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1

	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1

* end at 508 cycles
* We need to do two VSync pulses per line
* Line 5
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1

	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1

* end at 508 cycles
* We need to do two VSync pulses per line
* Line 6
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1

	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1

* end at 508 cycles
* Trailing Equalization Pulses
* We need to do two equalization pulses per line
* Line 7
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
* 
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1	

* end at 508 cycles
* We need to do two equalization pulses per line
* Line 8
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
* 
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1	

* end at 508 cycles
* We need to do two equalization pulses per line
* Line 9
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
* 
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ2	
	BCLR	TOF,TSC		; [4] reset interrupt flag

	LDHX	#TIRQ3	    	; Timer Overflow Vector: active video
	STHX	timer_vec
	MOV	#NVL,vcount ; number of vertical lines: 10 test, 253 real sync
	MOV	#9,vline  ; number of lines used in Vertical Blanking Interval

	LDHX	#vlf1		; the first video line function for Field 1
	STHX	vlfunction

	JMP	TIRQRET
*======================
* Active Video: Field 1 begins on Line 10, ends on Line 262 1/2
* Real Horizontal Sync
TIRQ3:	

*---------
* HSYNC will be 4.77 us = 34 cycles
	BSET	CSYNC,PORTB	; [4]  SYNC on

	BCLR	TOF,TSC		; [4]  reset interrupt flag

	LDX	#9		; [2]
tq3a:	DBNZX	tq3a		; [3]
	NOP			; [1]
	BCLR	CSYNC,PORTB	; [4]  SYNC off
* ---------------------------------------------------
* Check Vertical Count
	DBNZ	vcount,tq3c

* We have reached the end of active video. Next, start VBI
	LDHX	#TIRQ4	     ; Timer Overflow Vector: Start VBI next time
	STHX	timer_vec

	JMP	TIRQRET

tq3c:	JSR	Do_Video
	JMP	TIRQRET
*========================================================================
* Active Video: end of Field 1
* Start Field 2 VBI 
* Do 1/2 Horizontal line and 1 Equalizing pulse
* Real Horizontal Sync
TIRQ4:	

* Line 262
*---------
* HSYNC will be 4.77 us = 38 cycles
	BSET	CSYNC,PORTB	; [4]  SYNC on

	BCLR	TOF,TSC		; [4]  reset interrupt flag

	LDX	#9		; [2]
tq4a:	DBNZX	tq4a		; [3]
	NOP			; [1]
	BCLR	CSYNC,PORTB	; [4]  SYNC off
* ---------------------------------------------------
* Bring cycle count from CSYNC to 254 (508/2)
	LDX	#70
tq4b:	DBNZX	tq4b
*---------
* Do 1 Equalizing Pulse
* Do the pulse and get ready for the next step
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ2

	LDHX	#TIRQ5	    ; Timer Overflow Vector: active video
	STHX	timer_vec

	JMP	TIRQRET
*======================
* Field 2 VBI: Equalization Pulses
TIRQ5:	
* Line 263
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
 
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1	

* end at 508 cycles
* Line 264
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
 
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1	

* end at 508 cycles
* Line 265
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1

* end at 254 cycles (6 EQ pulses)
* Field 2 VBI: Vertical Sync Pulses
* We need to do two VSync pulses per line
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1

* Line 266
* end at 508 cycles
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1

* end at 254 cycles
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1

* Line 267
* end at 508 cycles
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1

	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1

* end at 508 cycles
* Line 268
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1

* end at 254 cycles
* Trailing Equalization Pulses
* We need to do two equalization pulses per line
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
 
* end at 508 cycles
* Line 269
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1	

	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
 
* end at 508 cycles
* Line 270
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1	

	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
 
* end at 508 cycles
* Line 271
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ2	
	BCLR	TOF,TSC		; [4] reset interrupt flag

* Start of Field 2 Video
	LDX	#7		; [2]	add 4+4+4
tq5a:	DBNZX	tq5a		; [3]
	NOP			; [1]

	BCLR	CSYNC,PORTB	; [4]  SYNC off

* Wait 
	LDX	#6	  ; [2]
tq5b:	DBNZX	tq5b	  ; [3]

* End of trailing equalization pulses
* Line 271 1/2 end early to give cycles to doing the clock
* This only affects the VBLANK signal
*	BCLR	VBLANK,PORTB	; oscilloscope trigger low
	MOV	#9,vline
	JSR	Get_CH		; get character addresses into display list

* Line 271 1/2 or so
*---------
* Start Field 2 Active Video, which ends in an integer number of H lines
* and begins Field 1 VBI
	LDHX	#TIRQ1		; Timer Overflow Vector
	STHX	timer_vec

	MOV	#NVL,vcount ; number of vertical lines: 10 test, 253 real sync
	MOV	#9,vline  ; number of lines used in Vertical Blanking Interval

	LDHX	#vlf1		; the first video line function for Field 2
	STHX	vlfunction

	JMP	TIRQRET
*============================================================================
* Equalization Pulse
EQ1:	
*	BSET	CSYNC,PORTB	; [4]  SYNC on

	LDX	#3		; [2]
eq1a:	DBNZX	eq1a		; [3]

	BCLR	CSYNC,PORTB	; [4]  SYNC off

* Wait 
	LDX	#74		; [2]
eq1b:	DBNZX	eq1b	   	; [3]
	NOP
	NOP

	RTS			; [4]
*==========================================================================
* Equalization Pulse
EQ2:	
*	BSET	CSYNC,PORTB	; [4]  SYNC on

	LDX	#3		; [2]
eq2a:	DBNZX	eq2a		; [3]
	BCLR	CSYNC,PORTB	; [4]  SYNC off

	RTS			; [4]
*==========================================================================
* VSync pulse
VS1:	
*	BSET	CSYNC,PORTB	; [4]  SYNC on

	LDX	#69
vs1a:	DBNZX	vs1a	   	; [3]	

	BCLR	CSYNC,PORTB	; [4]  SYNC off
	
	LDX	#8		; [3]
vs1b:	DBNZX	vs1b	   	; [3]	
	NOP			; [1]
	NOP			; [1]
	RTS
*==========================================================================
***************************************************************************
*==========================================================================
* Wait 1000 us
*   cycles = 19 + 8*X ; X = (cycles-19)/8
*
* At 8.0 MHz, we need 1000*8.0 = 8000 cycles
* xval = (time*BClock -19)/8
* time = (xval*8 +19)/BClock
* For BClock = 8.0 MHz, xval = 997
*			JSR		4
WAIT1ms:
        PSHX		;		2
        PSHH		;		2

	LDHX	#997	; 		3

wt1ms	AIX	#-1	;		2
	CPHX	#0000	;		3
	BNE	wt1ms   ;		3

	PULH		;		2
	PULX		;		2
	RTS		;		4
*=============================================================================
TEST1:	BSET	CSYNC,PORTB
	JSR	WAIT1ms
	BCLR	CSYNC,PORTB
	JSR	WAIT1ms
	BRA	TEST1
*=============================================================================
CHTABLE	FDB	ch_0,ch_1,ch_2,ch_3,ch_4,ch_5,ch_6,ch_7,ch_8,ch_9

* Video Line Functions
Do_Video:
	LDA	vline
	INC	vline		; for next time

	LDHX	vlfunction
	JMP	,X

* vlf1 - line 10: Read and debounce the switches, set the line trigger values
* vlf2 - line 11: Convert line1 value to bcd
* vlf3 - line 12: Convert line2 value to bcd
* vlf4 - line 100: set up group 1 characters
* vlf5 - do group 1 characters 
* vlf6 - line 150: set up group 2 characters
* vlf7 - do group 2 characters 
* vlf8 - send first byte of new line data through the link
* vlf9 - send second byte of new line data through the link
* vlf10 - do nothing
*-------------------------------------------------------------
* Line 10
* Read and debounce the switches, set the line1 and line2 values
vlf1:	CMP	#10
	BNE	v1out

	JSR	Do_Switches	; read the switches and set the line values

	LDHX	#vlf2		; next function
	STHX	vlfunction

v1out:	RTS
*===========================
* Line 11
* Convert line1 value to bcd
vlf2:	LDHX	line1
	STHX	bin

	JSR	bin_bcd

	LDHX	bcd
	STHX	line1bcd

	LDHX	#vlf3		; next function
	STHX	vlfunction

v2out:	RTS
*===========================
* Line 12
* Convert line2 value to bcd
vlf3:	LDHX	line2
	STHX	bin

	JSR	bin_bcd

	LDHX	bcd
	STHX	line2bcd

	LDHX	#vlf4		; next function
	STHX	vlfunction

v3out:	RTS
*===========================
vlf4:
* ACC contains vline number
* Group 1 characters
* Set up characters for next line
	CMP	#100		; start characters at line 101
	BNE	v4out		; do characters?

	MOV	#8,chline

*	LDHX	#ch_0		; for testing
	LDHX	ch0adstor
	STHX	ch0address

*	LDHX	#ch_1		; for testing
	LDHX	ch1adstor
	STHX	ch1address

*	LDHX	#ch_2		; for testing
	LDHX	ch2adstor
	STHX	ch2address

	LDHX	#ch_arw		; arrow
	STHX	ch6address

	MOV	#4,chtimes

	LDHX	#vlf5			; start it and display it next time
	STHX	vlfunction

v4out:	RTS
*-------------------------------------------------------------
* Display the Group 1 characters
vlf5:	LDA	chline
	BNE	v5a		; still going

	LDHX	#vlf6		; next group
	STHX	vlfunction

	BRA	v5out

* do the characters
* Wait a little
v5a:	LDX	#20		; [2]
v5b:	DBNZX	v5b  		; [3]	

	LDHX	ch0address
	JSR	,X

	LDHX	ch1address
	JSR	,X

	LDHX	ch2address
	JSR	,X

	BRSET	0,fsel,v5c	; display the arrow?
	BRSET	C2En,PORTB,v5c	; but not if in single channel mode

	LDHX	ch6address
	JSR	,X

v5c:	DEC	chtimes
	BNE	v5out		; repeat the line

	DEC	chline		; next line

	LDHX	ch0address
	AIX	#19		; a line takes 19 bytes
	STHX	ch0address

	LDHX	ch1address
	AIX	#19		; a line takes 19 bytes
	STHX	ch1address

	LDHX	ch2address
	AIX	#19		; a line takes 19 bytes
	STHX	ch2address

	LDHX	ch6address
	AIX	#19		; a line takes 19 bytes
	STHX	ch6address

v5d:	MOV	#4,chtimes

v5out:	RTS
*-------------------------------------------------------------
vlf6:

* Are we doing one channel or two?
	BRCLR	C2En,PORTB,v6a		; Channel 2 enabled?

* One channel
	LDHX	#vlf8			; skip channel 2
	STHX	vlfunction

* ACC contains vline number
* Group 2 characters
* Set up characters for next line
v6a:	CMP	#150		; group 2 setup characters?
	BNE	v6out		; do characters?

	MOV	#8,chline

	LDHX	ch3adstor
	STHX	ch3address

	LDHX	ch4adstor
	STHX	ch4address

	LDHX	ch5adstor
	STHX	ch5address

	LDHX	#ch_arw		; arrow
	STHX	ch6address

	MOV	#4,chtimes

	LDHX	#vlf7			; start it and display it next time
	STHX	vlfunction

v6out:	RTS
*-------------------------------------------------------------
* Display the group 2 characters
vlf7:	LDA	chline
	BNE	v7a		; still going

	LDHX	#vlf8		; next group
	STHX	vlfunction

	BRA	v7out

* do the characters
* Wait a little
v7a:	LDX	#20		; [2]
v7b:	DBNZX	v7b  		; [3]	

	LDHX	ch3address
	JSR	,X

	LDHX	ch4address
	JSR	,X

	LDHX	ch5address
	JSR	,X

	BRCLR	0,fsel,v7c	; display the arrow?
	BRSET	C2En,PORTB,v7c1	; create same delay as in channel 1

v7c1:	LDHX	ch6address
	JSR	,X

v7c:	DEC	chtimes
	BNE	v7out		; repeat the line

	DEC	chline		; next line

	LDHX	ch3address
	AIX	#19		; a line takes 19 bytes
	STHX	ch3address

	LDHX	ch4address
	AIX	#19		; a line takes 19 bytes
	STHX	ch4address

	LDHX	ch5address
	AIX	#19		; a line takes 19 bytes
	STHX	ch5address

	LDHX	ch6address
	AIX	#19		; a line takes 19 bytes
	STHX	ch6address

	MOV	#4,chtimes

v7out:	RTS
*-------------------------------------------------------------
* vlf8 - send new line data through the link
vlf8:	BRSET	7,fsel,v8a

* No new data
	LDHX	#vlf10		; next
	STHX	vlfunction
	BRA	v8ret

* We have new data to send
v8a:	BCLR	7,fsel			; reset the flag

	BRSET	0,fsel,v8b		; which line data to send

* line 1
	LDHX	line1
	STHX	linkdata
	BCLR	2,linkdata	      ; d2 = line sel, d1,d0 msb of line data

	BRA	v8c		      ; send it

* line 2
v8b:	LDHX	line2
	STHX	linkdata
	BSET	2,linkdata	      ; d2 = line sel, d1,d0 msb of line data

v8c:	LDA	linkdata
	JSR	Link_Send	      ; send the data through the link

v8out:	LDHX	#vlf9		; next
	STHX	vlfunction

v8ret:	RTS
*-------------------------------------------------------------
* vlf9 - send second byte of new data through the link
vlf9:	LDA	linkdata+1
	JSR	Link_Send	      ; send the data through the link

	LDHX	#vlf10		      ; next
	STHX	vlfunction

	RTS
*-------------------------------------------------------------
* No more to do
vlf10:	RTS
*==========================================================================
* Get the character addresses ready
* Done during VBI
Get_CH:	

* line 1 trigger value
	LDHX	line1bcd

	CLRH

* line 1 first character 0V xx
	LDA	line1bcd
	AND	#$0F
	ASLA
	CMPA	#18	    ; 9 is max (we have already shifted the data by 2)
	BLE	gch1		
	CLRA
gch1:	TAX

	LDA	CHTABLE,X
	STA	ch0adstor
	LDA	CHTABLE+1,X
	STA	ch0adstor+1

* line 1 second character 0x Vx
	LDA	line1bcd+1
	PSHA			; save it
	AND	#$F0
	LSRA
	LSRA
	LSRA
	CMPA	#18	    ; 9 is max (we have already shifted the data by 2)
	BLE	gch2
	CLRA
gch2:	TAX

	LDA	CHTABLE,X
	STA	ch1adstor
	LDA	CHTABLE+1,X
	STA	ch1adstor+1

* line 1 third character 0x xV
	PULA
	AND	#$0F
	ASLA
	CMPA	#18	    ; 9 is max (we have already shifted the data by 2)
	BLE	gch3		
	CLRA
gch3:	TAX

	LDA	CHTABLE,X
	STA	ch2adstor
	LDA	CHTABLE+1,X
	STA	ch2adstor+1
*---------
* line 2 trigger value
	CLRH

* line 2 first character 0V xx
	LDA	line2bcd
	AND	#$0F
	ASLA
	CMPA	#18	    ; 9 is max (we have already shifted the data by 2)
	BLE	gch4		
	CLRA
gch4:	TAX

	LDA	CHTABLE,X
	STA	ch3adstor
	LDA	CHTABLE+1,X
	STA	ch3adstor+1

* line 2 second character 0x Vx
	LDA	line2bcd+1
	PSHA			; save it
	AND	#$F0
	LSRA
	LSRA
	LSRA
	CMPA	#18	    ; 9 is max (we have already shifted the data by 2)
	BLE	gch5
	CLRA
gch5:	TAX

	LDA	CHTABLE,X
	STA	ch4adstor
	LDA	CHTABLE+1,X
	STA	ch4adstor+1

* line 2 third character 0x xV
	PULA
	AND	#$0F
	ASLA
	CMPA	#18	    ; 9 is max (we have already shifted the data by 2)
	BLE	gch6		
	CLRA
gch6:	TAX

	LDA	CHTABLE,X
	STA	ch5adstor
	LDA	CHTABLE+1,X
	STA	ch5adstor+1

	RTS
*==========================================================================
* Read and debounce the switches, set the clock as required
Do_Switches:

* Do the Increment Switch
dsw1:	LDA	sw1		
	BRSET	IncSW,PORTB,dsw1a  ; sets the carry to the state of switch
dsw1a:	ROLA			; shifts Carry into sw1
	STA	sw1
	AND	#$07
	BEQ	dsw1b		; do we have all '0's in low bits?

	CMP	#$07		; do we have all '1's in low bits?
	BNE	dsw2		; next switch
	STA	sw1+1		; debounced switch = off
	CLR	sw1+2		; reset the switch duration counter
	BRA	dsw2		; switch is not pressed

* switch is pressed
dsw1b:	LDX	sw1+2		; increment the switch duration counter
	INCX
	CMPX	#30		; duration is 30 frames
	BMI	dsw1f		; short time

* switch has been pressed for a long time
* increment minutes
	CLR	sw1+2		; reset the duration counter
	JSR	Inc_Tens
	BRA	dsw3		; only 1 switch at a time

dsw1f:	STX	sw1+2		; save the switch duration counter
	COMA
	EOR	sw1+1		; previous switch state
	AND	#$07
	BNE	dsw2		; same as last time

	CLR	sw1+1		; debounced switch = on
	
* we had a transition from off-to-on
* switch press positive transition
* switch has been pressed for a short time
* increment minutes
	LDA	#$01
	JSR	Inc_Ones
	BRA	dsw3
*-----
* Do the Decrement Switch
dsw2:	LDA	sw2
	BRSET	DecSW,PORTB,dsw2a  ; sets the carry to the state of switch
dsw2a:	ROLA			     ; shifts Carry into sw2
	STA	sw2
	AND	#$07
	BEQ	dsw2b		; do we have all '0's in low bits?

	CMP	#$07		; do we have all '1's in low bits?
	BNE	dsw3
	STA	sw2+1		; debounced switch = off
	CLR	sw2+2		; reset the switch duration counter
	BRA	dsw3		; switch is not pressed

* switch is pressed
dsw2b:	LDX	sw2+2		; increment the switch duration counter
	INCX
	CMPX	#30		; duration is 30 frames
	BMI	dsw2f		; short time

* switch has been pressed for a long time
* decrement minutes
	CLR	sw2+2		; reset the duration counter
	JSR	Dec_Tens
	BRA	dsw3

dsw2f:	STX	sw2+2
	COMA
	EOR	sw2+1		; previous switch state
	AND	#$07
	BNE	dsw3		; same as last time

	CLR	sw2+1		; debounced switch = on

* switch has been pressed for a short time
* increment minutes
* decrement minutes
	LDA	#$99		; add 100 - 1
	JSR	Dec_Ones
*-----
* Do the Switch to select whether to change line1 or line2
dsw3:	LDA	sw3
	BRSET	SelSW,PORTB,dsw3a  ; sets the carry to the state of switch
dsw3a:	ROLA			; shifts Carry into sw1
	STA	sw3
	CMP	#$80
	BNE	dsw4		; looking for an off-to-on transition

* we had a transition from off-to-on
* switch press positive transition
* If we are in two-channel mode, toggle the line select
	BRCLR	C2En,PORTB,dsw3b	; Channel 2 enabled?
	BCLR	0,fsel

* If we are in single channel mode, change the line to the other field
	LDHX	line1
	CPHX	#263
	BHI	cf2
* We are in field 1, so add 263 ($107)
	LDA	line1+1
	ADD	#$07
	STA	line1+1

	LDA	line1
	ADC	#$01
	STA	line1

	LDHX	line1
	CPHX	#526
	BMI	cf1

	LDHX	#$0009
	STHX	line1

cf1:	BSET	7,fsel		; we have changed the line data
	BRA	dsw4

* We are in field 2, so subtract 263 ($107)
cf2:	LDA	line1+1
	SUB	#$07
	STA	line1+1

	LDA	line1
	SBC	#$01
	STA	line1

	LDHX	#$0009
	CPHX	#0009
	BPL	cf3

	LDHX	#525
	STHX	line1

cf3:	BSET	7,fsel		; we have changed the line data
	BRA	dsw4

* Channel 2 enabled
dsw3b:	LDA	fsel
	EOR	#$01
	STA	fsel

* Check again in case we got here without a switch transition
dsw4:	BRCLR	C2En,PORTB,dsw4a	
	BCLR	0,fsel

dsw4a:	RTS
*==========================================================================
Inc_Ones:
	BRSET	0,fsel,IO2

	LDHX	line1
	AIX	#1
	STHX	line1

	CPHX	#526
	BMI	IOnout

	LDHX	#$0009
	STHX	line1

	BRA	IOnout

IO2:	LDHX	line2
	AIX	#1
	STHX	line2

	CPHX	#526
	BMI	IOnout

	LDHX	#$0009
	STHX	line2

IOnout:	BSET	7,fsel		; we have changed the line data
	RTS
*--------------------
Dec_Ones:
	BRSET	0,fsel,DO2

	LDHX	line1
	AIX	#-1
	STHX	line1

	CPHX	#0009
	BPL	DOnout

	LDHX	#525
	STHX	line1

	BRA	DOnout

DO2:	LDHX	line2
	AIX	#-1
	STHX	line2

	CPHX	#0009
	BPL	DOnout

	LDHX	#525
	STHX	line2

DOnout:	BSET	7,fsel		; we have changed the line data
	RTS
*--------------------
Inc_Tens:
	BRSET	0,fsel,IT2

	LDHX	line1
	AIX	#10
	STHX	line1

	CPHX	#526
	BMI	ITnout

	LDHX	#$0009
	STHX	line1

	BRA	ITnout

IT2:	LDHX	line2
	AIX	#10
	STHX	line2

	CPHX	#526
	BMI	ITnout

	LDHX	#$0009
	STHX	line2

ITnout:	BSET	7,fsel		; we have changed the line data
	RTS
*--------------------
Dec_Tens:
	BRSET	0,fsel,DT2

	LDHX	line1
	AIX	#-10
	STHX	line1

	CPHX	#0009
	BPL	DTnout

	LDHX	#525
	STHX	line1

	BRA	DTnout

DT2:	LDHX	line2
	AIX	#-10
	STHX	line2

	CPHX	#0009
	BPL	DTnout

	LDHX	#525
	STHX	line2

DTnout:	BSET	7,fsel		; we have changed the line data
	RTS
*==========================================================================
* Convert binary to bcd, 1023d max
* input from bin, bin+1; output in bcd, bcd+1
BIN_BCD:
	CLR     bcd+1
        CLR     bcd

* First convert bin+1 (0-255)
        LDX     #8
bbcd1:
        ROL     bin+1

	LDA	bcd+1
	ADC	bcd+1
	DAA
	STA	bcd+1

	LDA	bcd
	ADC	bcd
	DAA
	STA	bcd

        DBNZX   bbcd1

* If bin is 0, then we are done
	LDA	bin
	BEQ	bbout

* If it is $01, add 256d
	CMP	#$01
	BNE	bbcd2

	LDA	bcd+1
	ADD	#$56
	DAA
	STA	bcd+1

	LDA	bcd
	ADC	#$02
	DAA
	STA	bcd
	BRA	bbout

* If it is $02, add 512d
bbcd2:	CMP	#$02
	BNE	bbcd3

	LDA	bcd+1
	ADD	#$12
	DAA
	STA	bcd+1

	LDA	bcd
	ADC	#$05
	DAA
	STA	bcd
	BRA	bbout

* It should not get here since the maximum line number should be 525
bbcd3:

bbout:	RTS
*==========================================================================
* Send line data (in ACC) to MCU1 through the link
* Use Data and Clock, Data is set up before rising edge of Clock
* ___	         __     __     __     __     __     __     __     ________
*    \________d0/  \_d1/  \_d2/  \_d3/  \_d4/  \_d5/  \_d6/  \ d7/ 

Link_Send:

* Send the start bit
	CLRH
	BCLR	Link_Clk,PORTB	; [4]

* Give MCU1 time to recognize the first falling edge as an interrupt
	LDX	#20
ls1:	DBNZX	ls1		; [3]

	LDX	#8
lsloop:	LSRA			; [1]

	BCC	ls2		; [3]  send a '0'

* send a '1'
	BSET	Link_Dat,PORTB	; [4]
	BRA	ls3		; [3]

* send a '0'
ls2:	BCLR	Link_Dat,PORTB	; [4]
	BRA	ls3		; [3]

* Clock it
ls3:	BSET	Link_Clk,PORTB	; [4]

* wait 12 cycles to give MCU1 time to get the data
	BSET	Link_Clk,PORTB	; [4] waste cycles
	BSET	Link_Clk,PORTB	; [4] waste cycles
	BSET	Link_Clk,PORTB	; [4] waste cycles

	DECX	
	BEQ	lsout

	BCLR	Link_Clk,PORTB	; [4]	Begin the clock for anothe bit
	BRA	lsloop		; [3]

lsout:	RTS
*==========================================================================
	ORG	$F470
$INCLUDE 'chars2.h'
*==========================================================================
******************************************************************************
* NOP Table
******************************************************************************
*==========================================================================
* Put NOPs here
	ORG	NOPORG
$INCLUDE 'nops1.h'   ; 456 NOPs (NOPSHI and NOPSLO must match this address)
*==========================================================================
******************************************************************************
* Vector Table
	ORG	ADCVEC		; A/D conversion complete		
	FDB	UNUSED

	ORG	KBVEC	 	; Keyyboard Vector			
	FDB	UNUSED

	ORG	TIM_OF_VEC  	; Timer Overflow Vector		
	FDB	TIM_INT

	ORG	TIM_CH1_VEC 	; Timer Channel 1 Vector		
	FDB	UNUSED

	ORG	TIM_CH0_VEC	; Timer Channel 2 Vector		
	FDB	UNUSED

	ORG	IRQ_VEC		; IRQ Vector				
	FDB	UNUSED

	ORG	SWI_VEC		; Software Interrupt			
	FDB	UNUSED

	ORG 	RESET_VEC
	FDB	START
*=============================================================================
